# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .payloads import (
    PayloadsResource,
    AsyncPayloadsResource,
    PayloadsResourceWithRawResponse,
    AsyncPayloadsResourceWithRawResponse,
    PayloadsResourceWithStreamingResponse,
    AsyncPayloadsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.content_scanning import content_scanning_create_params, content_scanning_update_params
from ...types.content_scanning.content_scanning_get_response import ContentScanningGetResponse
from ...types.content_scanning.content_scanning_create_response import ContentScanningCreateResponse
from ...types.content_scanning.content_scanning_update_response import ContentScanningUpdateResponse

__all__ = ["ContentScanningResource", "AsyncContentScanningResource"]


class ContentScanningResource(SyncAPIResource):
    @cached_property
    def payloads(self) -> PayloadsResource:
        return PayloadsResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ContentScanningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ContentScanningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContentScanningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ContentScanningResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        value: Literal["enabled", "disabled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningCreateResponse:
        """
        Update the Content Scanning status.

        Args:
          zone_id: Defines an identifier.

          value: The status value for Content Scanning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            body=maybe_transform({"value": value}, content_scanning_create_params.ContentScanningCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningCreateResponse], ResultWrapper[ContentScanningCreateResponse]),
        )

    def update(
        self,
        *,
        zone_id: str,
        value: Literal["enabled", "disabled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningUpdateResponse:
        """
        Update the Content Scanning status.

        Args:
          zone_id: Defines an identifier.

          value: The status value for Content Scanning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            body=maybe_transform({"value": value}, content_scanning_update_params.ContentScanningUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningUpdateResponse], ResultWrapper[ContentScanningUpdateResponse]),
        )

    def disable(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Disable Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/content-upload-scan/disable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def enable(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Enable Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/content-upload-scan/enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningGetResponse:
        """
        Retrieve the current status of Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningGetResponse], ResultWrapper[ContentScanningGetResponse]),
        )


class AsyncContentScanningResource(AsyncAPIResource):
    @cached_property
    def payloads(self) -> AsyncPayloadsResource:
        return AsyncPayloadsResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncContentScanningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContentScanningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContentScanningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncContentScanningResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        value: Literal["enabled", "disabled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningCreateResponse:
        """
        Update the Content Scanning status.

        Args:
          zone_id: Defines an identifier.

          value: The status value for Content Scanning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            body=await async_maybe_transform(
                {"value": value}, content_scanning_create_params.ContentScanningCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningCreateResponse], ResultWrapper[ContentScanningCreateResponse]),
        )

    async def update(
        self,
        *,
        zone_id: str,
        value: Literal["enabled", "disabled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningUpdateResponse:
        """
        Update the Content Scanning status.

        Args:
          zone_id: Defines an identifier.

          value: The status value for Content Scanning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            body=await async_maybe_transform(
                {"value": value}, content_scanning_update_params.ContentScanningUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningUpdateResponse], ResultWrapper[ContentScanningUpdateResponse]),
        )

    async def disable(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Disable Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/content-upload-scan/disable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def enable(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Enable Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/content-upload-scan/enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContentScanningGetResponse:
        """
        Retrieve the current status of Content Scanning.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/content-upload-scan/settings", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ContentScanningGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ContentScanningGetResponse], ResultWrapper[ContentScanningGetResponse]),
        )


class ContentScanningResourceWithRawResponse:
    def __init__(self, content_scanning: ContentScanningResource) -> None:
        self._content_scanning = content_scanning

        self.create = to_raw_response_wrapper(
            content_scanning.create,
        )
        self.update = to_raw_response_wrapper(
            content_scanning.update,
        )
        self.disable = to_raw_response_wrapper(
            content_scanning.disable,
        )
        self.enable = to_raw_response_wrapper(
            content_scanning.enable,
        )
        self.get = to_raw_response_wrapper(
            content_scanning.get,
        )

    @cached_property
    def payloads(self) -> PayloadsResourceWithRawResponse:
        return PayloadsResourceWithRawResponse(self._content_scanning.payloads)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._content_scanning.settings)


class AsyncContentScanningResourceWithRawResponse:
    def __init__(self, content_scanning: AsyncContentScanningResource) -> None:
        self._content_scanning = content_scanning

        self.create = async_to_raw_response_wrapper(
            content_scanning.create,
        )
        self.update = async_to_raw_response_wrapper(
            content_scanning.update,
        )
        self.disable = async_to_raw_response_wrapper(
            content_scanning.disable,
        )
        self.enable = async_to_raw_response_wrapper(
            content_scanning.enable,
        )
        self.get = async_to_raw_response_wrapper(
            content_scanning.get,
        )

    @cached_property
    def payloads(self) -> AsyncPayloadsResourceWithRawResponse:
        return AsyncPayloadsResourceWithRawResponse(self._content_scanning.payloads)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._content_scanning.settings)


class ContentScanningResourceWithStreamingResponse:
    def __init__(self, content_scanning: ContentScanningResource) -> None:
        self._content_scanning = content_scanning

        self.create = to_streamed_response_wrapper(
            content_scanning.create,
        )
        self.update = to_streamed_response_wrapper(
            content_scanning.update,
        )
        self.disable = to_streamed_response_wrapper(
            content_scanning.disable,
        )
        self.enable = to_streamed_response_wrapper(
            content_scanning.enable,
        )
        self.get = to_streamed_response_wrapper(
            content_scanning.get,
        )

    @cached_property
    def payloads(self) -> PayloadsResourceWithStreamingResponse:
        return PayloadsResourceWithStreamingResponse(self._content_scanning.payloads)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._content_scanning.settings)


class AsyncContentScanningResourceWithStreamingResponse:
    def __init__(self, content_scanning: AsyncContentScanningResource) -> None:
        self._content_scanning = content_scanning

        self.create = async_to_streamed_response_wrapper(
            content_scanning.create,
        )
        self.update = async_to_streamed_response_wrapper(
            content_scanning.update,
        )
        self.disable = async_to_streamed_response_wrapper(
            content_scanning.disable,
        )
        self.enable = async_to_streamed_response_wrapper(
            content_scanning.enable,
        )
        self.get = async_to_streamed_response_wrapper(
            content_scanning.get,
        )

    @cached_property
    def payloads(self) -> AsyncPayloadsResourceWithStreamingResponse:
        return AsyncPayloadsResourceWithStreamingResponse(self._content_scanning.payloads)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._content_scanning.settings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["DCVDelegationUUID"]


class DCVDelegationUUID(BaseModel):
    uuid: Optional[str] = None
    """The DCV Delegation unique identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from .instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)
from .workflows import (
    WorkflowsResource,
    AsyncWorkflowsResource,
    WorkflowsResourceWithRawResponse,
    AsyncWorkflowsResourceWithRawResponse,
    WorkflowsResourceWithStreamingResponse,
    AsyncWorkflowsResourceWithStreamingResponse,
)

__all__ = [
    "InstancesResource",
    "AsyncInstancesResource",
    "InstancesResourceWithRawResponse",
    "AsyncInstancesResourceWithRawResponse",
    "InstancesResourceWithStreamingResponse",
    "AsyncInstancesResourceWithStreamingResponse",
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "WorkflowsResource",
    "AsyncWorkflowsResource",
    "WorkflowsResourceWithRawResponse",
    "AsyncWorkflowsResourceWithRawResponse",
    "WorkflowsResourceWithStreamingResponse",
    "AsyncWorkflowsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["OriginCloudRegionBulkUpdateResponse", "Failed", "Succeeded"]


class Failed(BaseModel):
    """Result for a single item in a batch operation."""

    origin_ip: str
    """The origin IP address for this item."""

    error: Optional[str] = None
    """Error message explaining why the item failed. Present only on failed items."""

    region: Optional[str] = None
    """Cloud vendor region identifier.

    Present on succeeded items (the new value for upsert, the deleted value for
    delete).
    """

    vendor: Optional[str] = None
    """Cloud vendor identifier.

    Present on succeeded items (the new value for upsert, the deleted value for
    delete).
    """


class Succeeded(BaseModel):
    """Result for a single item in a batch operation."""

    origin_ip: str
    """The origin IP address for this item."""

    error: Optional[str] = None
    """Error message explaining why the item failed. Present only on failed items."""

    region: Optional[str] = None
    """Cloud vendor region identifier.

    Present on succeeded items (the new value for upsert, the deleted value for
    delete).
    """

    vendor: Optional[str] = None
    """Cloud vendor identifier.

    Present on succeeded items (the new value for upsert, the deleted value for
    delete).
    """


class OriginCloudRegionBulkUpdateResponse(BaseModel):
    """Response result for a batch origin cloud region operation."""

    failed: List[Failed]
    """Items that could not be applied, with error details."""

    succeeded: List[Succeeded]
    """Items that were successfully applied."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ResourceListParams"]


class ResourceListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier."""

    page: int
    """Page number."""

    per_page: int
    """Number of objects to return per page."""

    resource_type: Literal[
        "custom-ruleset",
        "gateway-policy",
        "gateway-destination-ip",
        "gateway-block-page-settings",
        "gateway-extended-email-matching",
        "idp-federation-grant",
    ]
    """Filter share resources by resource_type."""

    status: Literal["active", "deleting", "deleted"]
    """Filter share resources by status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ConnectivitySettingGetResponse"]


class ConnectivitySettingGetResponse(BaseModel):
    icmp_proxy_enabled: Optional[bool] = None
    """A flag to enable the ICMP proxy for the account network."""

    offramp_warp_enabled: Optional[bool] = None
    """A flag to enable WARP to WARP traffic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .mitigations import (
    MitigationsResource,
    AsyncMitigationsResource,
    MitigationsResourceWithRawResponse,
    AsyncMitigationsResourceWithRawResponse,
    MitigationsResourceWithStreamingResponse,
    AsyncMitigationsResourceWithStreamingResponse,
)
from .abuse_reports import (
    AbuseReportsResource,
    AsyncAbuseReportsResource,
    AbuseReportsResourceWithRawResponse,
    AsyncAbuseReportsResourceWithRawResponse,
    AbuseReportsResourceWithStreamingResponse,
    AsyncAbuseReportsResourceWithStreamingResponse,
)

__all__ = [
    "MitigationsResource",
    "AsyncMitigationsResource",
    "MitigationsResourceWithRawResponse",
    "AsyncMitigationsResourceWithRawResponse",
    "MitigationsResourceWithStreamingResponse",
    "AsyncMitigationsResourceWithStreamingResponse",
    "AbuseReportsResource",
    "AsyncAbuseReportsResource",
    "AbuseReportsResourceWithRawResponse",
    "AsyncAbuseReportsResourceWithRawResponse",
    "AbuseReportsResourceWithStreamingResponse",
    "AsyncAbuseReportsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from .phishguard import (
    PhishguardResource,
    AsyncPhishguardResource,
    PhishguardResourceWithRawResponse,
    AsyncPhishguardResourceWithRawResponse,
    PhishguardResourceWithStreamingResponse,
    AsyncPhishguardResourceWithStreamingResponse,
)

__all__ = [
    "ReportsResource",
    "AsyncReportsResource",
    "ReportsResourceWithRawResponse",
    "AsyncReportsResourceWithRawResponse",
    "ReportsResourceWithStreamingResponse",
    "AsyncReportsResourceWithStreamingResponse",
    "PhishguardResource",
    "AsyncPhishguardResource",
    "PhishguardResourceWithRawResponse",
    "AsyncPhishguardResourceWithRawResponse",
    "PhishguardResourceWithStreamingResponse",
    "AsyncPhishguardResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["BotFightModeConfigurationParam"]


class BotFightModeConfigurationParam(TypedDict, total=False):
    ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"]
    """Enable rule to block AI Scrapers and Crawlers.

    Please note the value `only_on_ad_pages` is currently not available for
    Enterprise customers.
    """

    cf_robots_variant: Literal["off", "policy_only"]
    """Specifies the Robots Access Control License variant to use."""

    content_bots_protection: Literal["block", "disabled"]
    """Enable rule to block content bots.

    When enabled, blocks automated traffic with low bot scores, excluding safe
    verified bot categories. Exceptions should be managed via skip rules.
    """

    crawler_protection: Literal["enabled", "disabled"]
    """Enable rule to punish AI Scrapers and Crawlers via a link maze."""

    enable_js: bool
    """Use lightweight, invisible JavaScript detections to improve Bot Management.

    [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).
    """

    fight_mode: bool
    """Whether to enable Bot Fight Mode."""

    is_robots_txt_managed: bool
    """Enable cloudflare managed robots.txt.

    If an existing robots.txt is detected, then managed robots.txt will be prepended
    to the existing robots.txt.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import TypedDict

from .ip_network_param import IPNetworkParam

__all__ = ["DOHEndpointParam"]


class DOHEndpointParam(TypedDict, total=False):
    enabled: bool
    """Indicate whether the DOH endpoint is enabled for this location."""

    networks: Optional[Iterable[IPNetworkParam]]
    """Specify the list of allowed source IP network ranges for this endpoint.

    When the list is empty, the endpoint allows all source IPs. The list takes
    effect only if the endpoint is enabled for this location.
    """

    require_token: bool
    """Specify whether the DOH endpoint requires user identity authentication."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.access.logs import access_request_list_params
from .....types.zero_trust.access.logs.access_request_list_response import AccessRequestListResponse

__all__ = ["AccessRequestsResource", "AsyncAccessRequestsResource"]


class AccessRequestsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccessRequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AccessRequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccessRequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AccessRequestsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        allowed_op: Literal["eq", "neq"] | Omit = omit,
        app_type_op: Literal["eq", "neq"] | Omit = omit,
        app_uid_op: Literal["eq", "neq"] | Omit = omit,
        country_code_op: Literal["eq", "neq"] | Omit = omit,
        direction: Literal["desc", "asc"] | Omit = omit,
        email: str | Omit = omit,
        email_exact: bool | Omit = omit,
        email_op: Literal["eq", "neq", "contains"] | Omit = omit,
        fields: str | Omit = omit,
        idp_op: Literal["eq", "neq"] | Omit = omit,
        limit: int | Omit = omit,
        non_identity_op: Literal["eq", "neq"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        rayid_op: Literal["eq", "neq"] | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        user_id: str | Omit = omit,
        user_id_op: Literal["eq", "neq"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AccessRequestListResponse]:
        """
        Gets a list of Access authentication audit logs for an account.

        Args:
          account_id: Identifier.

          allowed_op: Operator for the `allowed` filter.

          app_type_op: Operator for the `app_type` filter.

          app_uid_op: Operator for the `app_uid` filter.

          country_code_op: Operator for the `country_code` filter.

          direction: The chronological sorting order for the logs.

          email: Filter by user email. Match mode is controlled by `emailOp` (preferred) or the
              legacy `email_exact` flag.

              - Default (no `emailOp`, `email_exact=false` or unset): substring match —
                `email=@example.com` returns all events with that domain.
              - Exact match: set `emailOp=eq` (preferred) or `email_exact=true` — e.g.
                `email=user@example.com&email_exact=true` returns only that user.
              - Explicit substring match: set `emailOp=contains` (without `email_exact=true`).
                When both are set, `email_exact=true` takes precedence and the match is exact.
              - Exclusion: set `emailOp=neq`. With `email_exact=true` this is an exact-value
                exclusion; without it, a fuzzy substring exclusion.

          email_exact: When true, `email` is matched exactly instead of substring matching.

          email_op: Operator for the `email` filter. `contains` performs a substring
              (case-sensitive) match. When `email_exact=true` is also set, `email_exact` takes
              precedence and `contains` is ignored.

          fields: Comma-separated list of fields to include in the response. When omitted, all
              fields are returned.

          idp_op: Operator for the `idp` filter.

          limit: The maximum number of log entries to retrieve.

          non_identity_op: Operator for the `non_identity` filter.

          page: Page number of results.

          per_page: Number of results per page.

          rayid_op: Operator for the `ray_id` filter.

          since: The earliest event timestamp to query.

          until: The latest event timestamp to query.

          user_id: Deprecated. Accepted for backward compatibility but no longer applied as a
              filter. Use `email` instead.

          user_id_op: Deprecated. Accepted for backward compatibility but no longer applied as a
              filter (the `user_id` parameter is itself deprecated).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/access/logs/access_requests", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "allowed_op": allowed_op,
                        "app_type_op": app_type_op,
                        "app_uid_op": app_uid_op,
                        "country_code_op": country_code_op,
                        "direction": direction,
                        "email": email,
                        "email_exact": email_exact,
                        "email_op": email_op,
                        "fields": fields,
                        "idp_op": idp_op,
                        "limit": limit,
                        "non_identity_op": non_identity_op,
                        "page": page,
                        "per_page": per_page,
                        "rayid_op": rayid_op,
                        "since": since,
                        "until": until,
                        "user_id": user_id,
                        "user_id_op": user_id_op,
                    },
                    access_request_list_params.AccessRequestListParams,
                ),
                post_parser=ResultWrapper[Optional[AccessRequestListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AccessRequestListResponse]], ResultWrapper[AccessRequestListResponse]),
        )


class AsyncAccessRequestsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccessRequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccessRequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccessRequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAccessRequestsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        allowed_op: Literal["eq", "neq"] | Omit = omit,
        app_type_op: Literal["eq", "neq"] | Omit = omit,
        app_uid_op: Literal["eq", "neq"] | Omit = omit,
        country_code_op: Literal["eq", "neq"] | Omit = omit,
        direction: Literal["desc", "asc"] | Omit = omit,
        email: str | Omit = omit,
        email_exact: bool | Omit = omit,
        email_op: Literal["eq", "neq", "contains"] | Omit = omit,
        fields: str | Omit = omit,
        idp_op: Literal["eq", "neq"] | Omit = omit,
        limit: int | Omit = omit,
        non_identity_op: Literal["eq", "neq"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        rayid_op: Literal["eq", "neq"] | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        user_id: str | Omit = omit,
        user_id_op: Literal["eq", "neq"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AccessRequestListResponse]:
        """
        Gets a list of Access authentication audit logs for an account.

        Args:
          account_id: Identifier.

          allowed_op: Operator for the `allowed` filter.

          app_type_op: Operator for the `app_type` filter.

          app_uid_op: Operator for the `app_uid` filter.

          country_code_op: Operator for the `country_code` filter.

          direction: The chronological sorting order for the logs.

          email: Filter by user email. Match mode is controlled by `emailOp` (preferred) or the
              legacy `email_exact` flag.

              - Default (no `emailOp`, `email_exact=false` or unset): substring match —
                `email=@example.com` returns all events with that domain.
              - Exact match: set `emailOp=eq` (preferred) or `email_exact=true` — e.g.
                `email=user@example.com&email_exact=true` returns only that user.
              - Explicit substring match: set `emailOp=contains` (without `email_exact=true`).
                When both are set, `email_exact=true` takes precedence and the match is exact.
              - Exclusion: set `emailOp=neq`. With `email_exact=true` this is an exact-value
                exclusion; without it, a fuzzy substring exclusion.

          email_exact: When true, `email` is matched exactly instead of substring matching.

          email_op: Operator for the `email` filter. `contains` performs a substring
              (case-sensitive) match. When `email_exact=true` is also set, `email_exact` takes
              precedence and `contains` is ignored.

          fields: Comma-separated list of fields to include in the response. When omitted, all
              fields are returned.

          idp_op: Operator for the `idp` filter.

          limit: The maximum number of log entries to retrieve.

          non_identity_op: Operator for the `non_identity` filter.

          page: Page number of results.

          per_page: Number of results per page.

          rayid_op: Operator for the `ray_id` filter.

          since: The earliest event timestamp to query.

          until: The latest event timestamp to query.

          user_id: Deprecated. Accepted for backward compatibility but no longer applied as a
              filter. Use `email` instead.

          user_id_op: Deprecated. Accepted for backward compatibility but no longer applied as a
              filter (the `user_id` parameter is itself deprecated).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/access/logs/access_requests", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "allowed_op": allowed_op,
                        "app_type_op": app_type_op,
                        "app_uid_op": app_uid_op,
                        "country_code_op": country_code_op,
                        "direction": direction,
                        "email": email,
                        "email_exact": email_exact,
                        "email_op": email_op,
                        "fields": fields,
                        "idp_op": idp_op,
                        "limit": limit,
                        "non_identity_op": non_identity_op,
                        "page": page,
                        "per_page": per_page,
                        "rayid_op": rayid_op,
                        "since": since,
                        "until": until,
                        "user_id": user_id,
                        "user_id_op": user_id_op,
                    },
                    access_request_list_params.AccessRequestListParams,
                ),
                post_parser=ResultWrapper[Optional[AccessRequestListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AccessRequestListResponse]], ResultWrapper[AccessRequestListResponse]),
        )


class AccessRequestsResourceWithRawResponse:
    def __init__(self, access_requests: AccessRequestsResource) -> None:
        self._access_requests = access_requests

        self.list = to_raw_response_wrapper(
            access_requests.list,
        )


class AsyncAccessRequestsResourceWithRawResponse:
    def __init__(self, access_requests: AsyncAccessRequestsResource) -> None:
        self._access_requests = access_requests

        self.list = async_to_raw_response_wrapper(
            access_requests.list,
        )


class AccessRequestsResourceWithStreamingResponse:
    def __init__(self, access_requests: AccessRequestsResource) -> None:
        self._access_requests = access_requests

        self.list = to_streamed_response_wrapper(
            access_requests.list,
        )


class AsyncAccessRequestsResourceWithStreamingResponse:
    def __init__(self, access_requests: AsyncAccessRequestsResource) -> None:
        self._access_requests = access_requests

        self.list = async_to_streamed_response_wrapper(
            access_requests.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Any, cast
from typing_extensions import Literal

import httpx

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.firewall.waf import package_list_params
from .....types.firewall.waf.package_get_response import PackageGetResponse

__all__ = ["PackagesResource", "AsyncPackagesResource"]


class PackagesResource(SyncAPIResource):
    @cached_property
    def groups(self) -> GroupsResource:
        return GroupsResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> PackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PackagesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["name"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[object]:
        """
        Fetches WAF packages for a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          direction: The direction used to sort returned packages.

          match: When set to `all`, all the search requirements must match. When set to `any`,
              only one of the search requirements has to match.

          name: The name of the WAF package.

          order: The field used to sort returned packages.

          page: The page number of paginated results.

          per_page: The number of packages per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/waf/packages", zone_id=zone_id),
            page=SyncV4PagePaginationArray[object],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    package_list_params.PackageListParams,
                ),
            ),
            model=object,
        )

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        package_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageGetResponse:
        """
        Fetches the details of a WAF package.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          package_id: Defines a package identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        return cast(
            PackageGetResponse,
            self._get(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}", zone_id=zone_id, package_id=package_id
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, PackageGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncPackagesResource(AsyncAPIResource):
    @cached_property
    def groups(self) -> AsyncGroupsResource:
        return AsyncGroupsResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPackagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPackagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPackagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPackagesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["name"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[object, AsyncV4PagePaginationArray[object]]:
        """
        Fetches WAF packages for a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          direction: The direction used to sort returned packages.

          match: When set to `all`, all the search requirements must match. When set to `any`,
              only one of the search requirements has to match.

          name: The name of the WAF package.

          order: The field used to sort returned packages.

          page: The page number of paginated results.

          per_page: The number of packages per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/waf/packages", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[object],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    package_list_params.PackageListParams,
                ),
            ),
            model=object,
        )

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        package_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PackageGetResponse:
        """
        Fetches the details of a WAF package.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          package_id: Defines a package identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        return cast(
            PackageGetResponse,
            await self._get(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}", zone_id=zone_id, package_id=package_id
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, PackageGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class PackagesResourceWithRawResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                packages.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                packages.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def groups(self) -> GroupsResourceWithRawResponse:
        return GroupsResourceWithRawResponse(self._packages.groups)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._packages.rules)


class AsyncPackagesResourceWithRawResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                packages.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                packages.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithRawResponse:
        return AsyncGroupsResourceWithRawResponse(self._packages.groups)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._packages.rules)


class PackagesResourceWithStreamingResponse:
    def __init__(self, packages: PackagesResource) -> None:
        self._packages = packages

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                packages.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                packages.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def groups(self) -> GroupsResourceWithStreamingResponse:
        return GroupsResourceWithStreamingResponse(self._packages.groups)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._packages.rules)


class AsyncPackagesResourceWithStreamingResponse:
    def __init__(self, packages: AsyncPackagesResource) -> None:
        self._packages = packages

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                packages.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                packages.get,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithStreamingResponse:
        return AsyncGroupsResourceWithStreamingResponse(self._packages.groups)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._packages.rules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .advanced_tcp_protection.advanced_tcp_protection import (
    AdvancedTCPProtectionResource,
    AsyncAdvancedTCPProtectionResource,
    AdvancedTCPProtectionResourceWithRawResponse,
    AsyncAdvancedTCPProtectionResourceWithRawResponse,
    AdvancedTCPProtectionResourceWithStreamingResponse,
    AsyncAdvancedTCPProtectionResourceWithStreamingResponse,
)

__all__ = ["DDoSProtectionResource", "AsyncDDoSProtectionResource"]


class DDoSProtectionResource(SyncAPIResource):
    @cached_property
    def advanced_tcp_protection(self) -> AdvancedTCPProtectionResource:
        return AdvancedTCPProtectionResource(self._client)

    @cached_property
    def with_raw_response(self) -> DDoSProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DDoSProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DDoSProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DDoSProtectionResourceWithStreamingResponse(self)


class AsyncDDoSProtectionResource(AsyncAPIResource):
    @cached_property
    def advanced_tcp_protection(self) -> AsyncAdvancedTCPProtectionResource:
        return AsyncAdvancedTCPProtectionResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDDoSProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDDoSProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDDoSProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDDoSProtectionResourceWithStreamingResponse(self)


class DDoSProtectionResourceWithRawResponse:
    def __init__(self, ddos_protection: DDoSProtectionResource) -> None:
        self._ddos_protection = ddos_protection

    @cached_property
    def advanced_tcp_protection(self) -> AdvancedTCPProtectionResourceWithRawResponse:
        return AdvancedTCPProtectionResourceWithRawResponse(self._ddos_protection.advanced_tcp_protection)


class AsyncDDoSProtectionResourceWithRawResponse:
    def __init__(self, ddos_protection: AsyncDDoSProtectionResource) -> None:
        self._ddos_protection = ddos_protection

    @cached_property
    def advanced_tcp_protection(self) -> AsyncAdvancedTCPProtectionResourceWithRawResponse:
        return AsyncAdvancedTCPProtectionResourceWithRawResponse(self._ddos_protection.advanced_tcp_protection)


class DDoSProtectionResourceWithStreamingResponse:
    def __init__(self, ddos_protection: DDoSProtectionResource) -> None:
        self._ddos_protection = ddos_protection

    @cached_property
    def advanced_tcp_protection(self) -> AdvancedTCPProtectionResourceWithStreamingResponse:
        return AdvancedTCPProtectionResourceWithStreamingResponse(self._ddos_protection.advanced_tcp_protection)


class AsyncDDoSProtectionResourceWithStreamingResponse:
    def __init__(self, ddos_protection: AsyncDDoSProtectionResource) -> None:
        self._ddos_protection = ddos_protection

    @cached_property
    def advanced_tcp_protection(self) -> AsyncAdvancedTCPProtectionResourceWithStreamingResponse:
        return AsyncAdvancedTCPProtectionResourceWithStreamingResponse(self._ddos_protection.advanced_tcp_protection)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["SCIMResource", "AsyncSCIMResource"]


class SCIMResource(SyncAPIResource):
    @cached_property
    def groups(self) -> GroupsResource:
        return GroupsResource(self._client)

    @cached_property
    def users(self) -> UsersResource:
        return UsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> SCIMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SCIMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SCIMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SCIMResourceWithStreamingResponse(self)


class AsyncSCIMResource(AsyncAPIResource):
    @cached_property
    def groups(self) -> AsyncGroupsResource:
        return AsyncGroupsResource(self._client)

    @cached_property
    def users(self) -> AsyncUsersResource:
        return AsyncUsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSCIMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSCIMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSCIMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSCIMResourceWithStreamingResponse(self)


class SCIMResourceWithRawResponse:
    def __init__(self, scim: SCIMResource) -> None:
        self._scim = scim

    @cached_property
    def groups(self) -> GroupsResourceWithRawResponse:
        return GroupsResourceWithRawResponse(self._scim.groups)

    @cached_property
    def users(self) -> UsersResourceWithRawResponse:
        return UsersResourceWithRawResponse(self._scim.users)


class AsyncSCIMResourceWithRawResponse:
    def __init__(self, scim: AsyncSCIMResource) -> None:
        self._scim = scim

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithRawResponse:
        return AsyncGroupsResourceWithRawResponse(self._scim.groups)

    @cached_property
    def users(self) -> AsyncUsersResourceWithRawResponse:
        return AsyncUsersResourceWithRawResponse(self._scim.users)


class SCIMResourceWithStreamingResponse:
    def __init__(self, scim: SCIMResource) -> None:
        self._scim = scim

    @cached_property
    def groups(self) -> GroupsResourceWithStreamingResponse:
        return GroupsResourceWithStreamingResponse(self._scim.groups)

    @cached_property
    def users(self) -> UsersResourceWithStreamingResponse:
        return UsersResourceWithStreamingResponse(self._scim.users)


class AsyncSCIMResourceWithStreamingResponse:
    def __init__(self, scim: AsyncSCIMResource) -> None:
        self._scim = scim

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithStreamingResponse:
        return AsyncGroupsResourceWithStreamingResponse(self._scim.groups)

    @cached_property
    def users(self) -> AsyncUsersResourceWithStreamingResponse:
        return AsyncUsersResourceWithStreamingResponse(self._scim.users)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tld_get_params import TLDGetParams as TLDGetParams
from .tld_get_response import TLDGetResponse as TLDGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .configuration.configuration import (
    ConfigurationResource,
    AsyncConfigurationResource,
    ConfigurationResourceWithRawResponse,
    AsyncConfigurationResourceWithRawResponse,
    ConfigurationResourceWithStreamingResponse,
    AsyncConfigurationResourceWithStreamingResponse,
)

__all__ = ["TokenValidationResource", "AsyncTokenValidationResource"]


class TokenValidationResource(SyncAPIResource):
    @cached_property
    def configuration(self) -> ConfigurationResource:
        return ConfigurationResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> TokenValidationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokenValidationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokenValidationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokenValidationResourceWithStreamingResponse(self)


class AsyncTokenValidationResource(AsyncAPIResource):
    @cached_property
    def configuration(self) -> AsyncConfigurationResource:
        return AsyncConfigurationResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTokenValidationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokenValidationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokenValidationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokenValidationResourceWithStreamingResponse(self)


class TokenValidationResourceWithRawResponse:
    def __init__(self, token_validation: TokenValidationResource) -> None:
        self._token_validation = token_validation

    @cached_property
    def configuration(self) -> ConfigurationResourceWithRawResponse:
        return ConfigurationResourceWithRawResponse(self._token_validation.configuration)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._token_validation.rules)


class AsyncTokenValidationResourceWithRawResponse:
    def __init__(self, token_validation: AsyncTokenValidationResource) -> None:
        self._token_validation = token_validation

    @cached_property
    def configuration(self) -> AsyncConfigurationResourceWithRawResponse:
        return AsyncConfigurationResourceWithRawResponse(self._token_validation.configuration)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._token_validation.rules)


class TokenValidationResourceWithStreamingResponse:
    def __init__(self, token_validation: TokenValidationResource) -> None:
        self._token_validation = token_validation

    @cached_property
    def configuration(self) -> ConfigurationResourceWithStreamingResponse:
        return ConfigurationResourceWithStreamingResponse(self._token_validation.configuration)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._token_validation.rules)


class AsyncTokenValidationResourceWithStreamingResponse:
    def __init__(self, token_validation: AsyncTokenValidationResource) -> None:
        self._token_validation = token_validation

    @cached_property
    def configuration(self) -> AsyncConfigurationResourceWithStreamingResponse:
        return AsyncConfigurationResourceWithStreamingResponse(self._token_validation.configuration)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._token_validation.rules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .logging_param import LoggingParam

__all__ = [
    "SetCacheSettingsRuleParam",
    "ActionParameters",
    "ActionParametersBrowserTTL",
    "ActionParametersCacheKey",
    "ActionParametersCacheKeyCustomKey",
    "ActionParametersCacheKeyCustomKeyCookie",
    "ActionParametersCacheKeyCustomKeyHeader",
    "ActionParametersCacheKeyCustomKeyHost",
    "ActionParametersCacheKeyCustomKeyQueryString",
    "ActionParametersCacheKeyCustomKeyQueryStringExclude",
    "ActionParametersCacheKeyCustomKeyQueryStringInclude",
    "ActionParametersCacheKeyCustomKeyUser",
    "ActionParametersCacheReserve",
    "ActionParametersEdgeTTL",
    "ActionParametersEdgeTTLStatusCodeTTL",
    "ActionParametersEdgeTTLStatusCodeTTLStatusCodeRange",
    "ActionParametersServeStale",
    "ActionParametersSharedDictionary",
    "ExposedCredentialCheck",
    "Ratelimit",
]


class ActionParametersBrowserTTL(TypedDict, total=False):
    """How long client browsers should cache the response.

    Cloudflare cache purge will not purge content cached on client browsers, so high browser TTLs may lead to stale content.
    """

    mode: Required[Literal["respect_origin", "bypass_by_default", "override_origin", "bypass"]]
    """The browser TTL mode."""

    default: int
    """The browser TTL (in seconds) if you choose the "override_origin" mode."""


class ActionParametersCacheKeyCustomKeyCookie(TypedDict, total=False):
    """Which cookies to include in the cache key."""

    check_presence: SequenceNotStr[str]
    """A list of cookies to check for the presence of.

    The presence of these cookies is included in the cache key.
    """

    include: SequenceNotStr[str]
    """A list of cookies to include in the cache key."""


class ActionParametersCacheKeyCustomKeyHeader(TypedDict, total=False):
    """Which headers to include in the cache key."""

    check_presence: SequenceNotStr[str]
    """A list of headers to check for the presence of.

    The presence of these headers is included in the cache key.
    """

    contains: Dict[str, SequenceNotStr[str]]
    """A mapping of header names to a list of values.

    If a header is present in the request and contains any of the values provided,
    its value is included in the cache key.
    """

    exclude_origin: bool
    """Whether to exclude the origin header in the cache key."""

    include: SequenceNotStr[str]
    """A list of headers to include in the cache key."""


class ActionParametersCacheKeyCustomKeyHost(TypedDict, total=False):
    """How to use the host in the cache key."""

    resolved: bool
    """Whether to use the resolved host in the cache key."""


class ActionParametersCacheKeyCustomKeyQueryStringExclude(TypedDict, total=False):
    """Which query string parameters to exclude from the cache key."""

    all: Literal[True]
    """Whether to exclude all query string parameters from the cache key."""

    list: SequenceNotStr[str]
    """A list of query string parameters to exclude from the cache key."""


class ActionParametersCacheKeyCustomKeyQueryStringInclude(TypedDict, total=False):
    """Which query string parameters to include in the cache key."""

    all: Literal[True]
    """Whether to include all query string parameters in the cache key."""

    list: SequenceNotStr[str]
    """A list of query string parameters to include in the cache key."""


class ActionParametersCacheKeyCustomKeyQueryString(TypedDict, total=False):
    """Which query string parameters to include in or exclude from the cache key."""

    exclude: ActionParametersCacheKeyCustomKeyQueryStringExclude
    """Which query string parameters to exclude from the cache key."""

    include: ActionParametersCacheKeyCustomKeyQueryStringInclude
    """Which query string parameters to include in the cache key."""


class ActionParametersCacheKeyCustomKeyUser(TypedDict, total=False):
    """How to use characteristics of the request user agent in the cache key."""

    device_type: bool
    """Whether to use the user agent's device type in the cache key."""

    geo: bool
    """Whether to use the user agents's country in the cache key."""

    lang: bool
    """Whether to use the user agent's language in the cache key."""


class ActionParametersCacheKeyCustomKey(TypedDict, total=False):
    """Which components of the request are included or excluded from the cache key."""

    cookie: ActionParametersCacheKeyCustomKeyCookie
    """Which cookies to include in the cache key."""

    header: ActionParametersCacheKeyCustomKeyHeader
    """Which headers to include in the cache key."""

    host: ActionParametersCacheKeyCustomKeyHost
    """How to use the host in the cache key."""

    query_string: ActionParametersCacheKeyCustomKeyQueryString
    """Which query string parameters to include in or exclude from the cache key."""

    user: ActionParametersCacheKeyCustomKeyUser
    """How to use characteristics of the request user agent in the cache key."""


class ActionParametersCacheKey(TypedDict, total=False):
    """
    Which components of the request are included in or excluded from the cache key Cloudflare uses to store the response in cache.
    """

    cache_by_device_type: bool
    """Whether to separate cached content based on the visitor's device type."""

    cache_deception_armor: bool
    """
    Whether to protect from web cache deception attacks, while allowing static
    assets to be cached.
    """

    custom_key: ActionParametersCacheKeyCustomKey
    """Which components of the request are included or excluded from the cache key."""

    ignore_query_strings_order: bool
    """
    Whether to treat requests with the same query parameters the same, regardless of
    the order those query parameters are in.
    """


class ActionParametersCacheReserve(TypedDict, total=False):
    """
    Settings to determine whether the request's response from origin is eligible for Cache Reserve (requires a Cache Reserve add-on plan).
    """

    eligible: Required[bool]
    """Whether Cache Reserve is enabled.

    If this is true and a request meets eligibility criteria, Cloudflare will write
    the resource to Cache Reserve.
    """

    minimum_file_size: int
    """The minimum file size eligible for storage in Cache Reserve."""


_ActionParametersEdgeTTLStatusCodeTTLStatusCodeRangeReservedKeywords = TypedDict(
    "_ActionParametersEdgeTTLStatusCodeTTLStatusCodeRangeReservedKeywords",
    {
        "from": int,
    },
    total=False,
)


class ActionParametersEdgeTTLStatusCodeTTLStatusCodeRange(
    _ActionParametersEdgeTTLStatusCodeTTLStatusCodeRangeReservedKeywords, total=False
):
    """A range of status codes to apply the TTL to."""

    to: int
    """The upper bound of the range."""


class ActionParametersEdgeTTLStatusCodeTTL(TypedDict, total=False):
    value: Required[int]
    """The time to cache the response for (in seconds).

    A value of 0 is equivalent to setting the cache control header with the value
    "no-cache". A value of -1 is equivalent to setting the cache control header with
    the value of "no-store".
    """

    status_code: int
    """A single status code to apply the TTL to."""

    status_code_range: ActionParametersEdgeTTLStatusCodeTTLStatusCodeRange
    """A range of status codes to apply the TTL to."""


class ActionParametersEdgeTTL(TypedDict, total=False):
    """How long the Cloudflare edge network should cache the response."""

    mode: Required[Literal["respect_origin", "bypass_by_default", "override_origin"]]
    """The edge TTL mode."""

    default: int
    """The edge TTL (in seconds) if you choose the "override_origin" mode."""

    status_code_ttl: Iterable[ActionParametersEdgeTTLStatusCodeTTL]
    """A list of TTLs to apply to specific status codes or status code ranges."""


class ActionParametersServeStale(TypedDict, total=False):
    """When to serve stale content from cache."""

    disable_stale_while_updating: bool
    """
    Whether Cloudflare should disable serving stale content while getting the latest
    content from the origin.
    """


class ActionParametersSharedDictionary(TypedDict, total=False):
    """Configuration for shared dictionary compression.

    When set, Cloudflare injects Use-As-Dictionary headers on matching cacheable responses.
    """

    match_pattern: Required[str]
    """URL pattern for the Use-As-Dictionary match field.

    This pattern specifies which URLs can use this response as a dictionary.
    """


class ActionParameters(TypedDict, total=False):
    """The parameters configuring the rule's action."""

    additional_cacheable_ports: Iterable[int]
    """A list of additional ports that caching should be enabled on."""

    browser_ttl: ActionParametersBrowserTTL
    """How long client browsers should cache the response.

    Cloudflare cache purge will not purge content cached on client browsers, so high
    browser TTLs may lead to stale content.
    """

    cache: bool
    """Whether the request's response from the origin is eligible for caching.

    Caching itself will still depend on the cache control header and your other
    caching configurations.
    """

    cache_key: ActionParametersCacheKey
    """
    Which components of the request are included in or excluded from the cache key
    Cloudflare uses to store the response in cache.
    """

    cache_reserve: ActionParametersCacheReserve
    """
    Settings to determine whether the request's response from origin is eligible for
    Cache Reserve (requires a Cache Reserve add-on plan).
    """

    edge_ttl: ActionParametersEdgeTTL
    """How long the Cloudflare edge network should cache the response."""

    origin_cache_control: bool
    """Whether Cloudflare will aim to strictly adhere to RFC 7234."""

    origin_error_page_passthru: bool
    """Whether to generate Cloudflare error pages for issues from the origin server."""

    read_timeout: int
    """
    A timeout value between two successive read operations to use for your origin
    server. Historically, the timeout value between two read options from Cloudflare
    to an origin server is 100 seconds. If you are attempting to reduce HTTP 524
    errors because of timeouts from an origin server, try increasing this timeout
    value.
    """

    respect_strong_etags: bool
    """Whether Cloudflare should respect strong ETag (entity tag) headers.

    If false, Cloudflare converts strong ETag headers to weak ETag headers.
    """

    serve_stale: ActionParametersServeStale
    """When to serve stale content from cache."""

    shared_dictionary: ActionParametersSharedDictionary
    """Configuration for shared dictionary compression.

    When set, Cloudflare injects Use-As-Dictionary headers on matching cacheable
    responses.
    """

    strip_etags: bool
    """Whether to strip ETag headers from the origin response before caching."""

    strip_last_modified: bool
    """Whether to strip Last-Modified headers from the origin response before caching."""

    strip_set_cookie: bool
    """Whether to strip Set-Cookie headers from the origin response before caching."""


class ExposedCredentialCheck(TypedDict, total=False):
    """Configuration for exposed credential checking."""

    password_expression: Required[str]
    """An expression that selects the password used in the credentials check."""

    username_expression: Required[str]
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(TypedDict, total=False):
    """An object configuring the rule's rate limit behavior."""

    characteristics: Required[SequenceNotStr[str]]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: Required[int]
    """Period in seconds over which the counter is being incremented."""

    counting_expression: str
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: int
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: int
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: bool
    """Whether counting is only performed when an origin is reached."""

    score_per_period: int
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: str
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class SetCacheSettingsRuleParam(TypedDict, total=False):
    id: str
    """The unique ID of the rule."""

    action: Literal["set_cache_settings"]
    """The action to perform when the rule matches."""

    action_parameters: ActionParameters
    """The parameters configuring the rule's action."""

    description: str
    """An informative description of the rule."""

    enabled: bool
    """Whether the rule should be executed."""

    exposed_credential_check: ExposedCredentialCheck
    """Configuration for exposed credential checking."""

    expression: str
    """The expression defining which traffic will match the rule."""

    logging: LoggingParam
    """An object configuring the rule's logging behavior."""

    ratelimit: Ratelimit
    """An object configuring the rule's rate limit behavior."""

    ref: str
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .outages import (
    OutagesResource,
    AsyncOutagesResource,
    OutagesResourceWithRawResponse,
    AsyncOutagesResourceWithRawResponse,
    OutagesResourceWithStreamingResponse,
    AsyncOutagesResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.radar import annotation_list_params
from ...._base_client import make_request_options
from ....types.radar.annotation_list_response import AnnotationListResponse

__all__ = ["AnnotationsResource", "AsyncAnnotationsResource"]


class AnnotationsResource(SyncAPIResource):
    @cached_property
    def outages(self) -> OutagesResource:
        return OutagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AnnotationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AnnotationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AnnotationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AnnotationsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        asn: int | Omit = omit,
        data_source: Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ]
        | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        event_type: Literal["EVENT", "GENERAL", "OUTAGE", "PARTIAL_PROJECTION", "PIPELINE", "TRAFFIC_ANOMALY"]
        | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AnnotationListResponse:
        """
        Retrieves the latest annotations.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          data_source: Filters results by data source.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          event_type: Filters results by event type.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/annotations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "data_source": data_source,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "event_type": event_type,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "offset": offset,
                        "origin": origin,
                    },
                    annotation_list_params.AnnotationListParams,
                ),
                post_parser=ResultWrapper[AnnotationListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AnnotationListResponse], ResultWrapper[AnnotationListResponse]),
        )


class AsyncAnnotationsResource(AsyncAPIResource):
    @cached_property
    def outages(self) -> AsyncOutagesResource:
        return AsyncOutagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAnnotationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAnnotationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAnnotationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAnnotationsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        asn: int | Omit = omit,
        data_source: Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ]
        | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        event_type: Literal["EVENT", "GENERAL", "OUTAGE", "PARTIAL_PROJECTION", "PIPELINE", "TRAFFIC_ANOMALY"]
        | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        location: str | Omit = omit,
        offset: int | Omit = omit,
        origin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AnnotationListResponse:
        """
        Retrieves the latest annotations.

        Args:
          asn: Filters results by Autonomous System. Specify a single Autonomous System Number
              (ASN) as integer.

          data_source: Filters results by data source.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          event_type: Filters results by event type.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify an alpha-2 location code.

          offset: Skips the specified number of objects before fetching the results.

          origin: Filters results by origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/annotations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "data_source": data_source,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "event_type": event_type,
                        "format": format,
                        "limit": limit,
                        "location": location,
                        "offset": offset,
                        "origin": origin,
                    },
                    annotation_list_params.AnnotationListParams,
                ),
                post_parser=ResultWrapper[AnnotationListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AnnotationListResponse], ResultWrapper[AnnotationListResponse]),
        )


class AnnotationsResourceWithRawResponse:
    def __init__(self, annotations: AnnotationsResource) -> None:
        self._annotations = annotations

        self.list = to_raw_response_wrapper(
            annotations.list,
        )

    @cached_property
    def outages(self) -> OutagesResourceWithRawResponse:
        return OutagesResourceWithRawResponse(self._annotations.outages)


class AsyncAnnotationsResourceWithRawResponse:
    def __init__(self, annotations: AsyncAnnotationsResource) -> None:
        self._annotations = annotations

        self.list = async_to_raw_response_wrapper(
            annotations.list,
        )

    @cached_property
    def outages(self) -> AsyncOutagesResourceWithRawResponse:
        return AsyncOutagesResourceWithRawResponse(self._annotations.outages)


class AnnotationsResourceWithStreamingResponse:
    def __init__(self, annotations: AnnotationsResource) -> None:
        self._annotations = annotations

        self.list = to_streamed_response_wrapper(
            annotations.list,
        )

    @cached_property
    def outages(self) -> OutagesResourceWithStreamingResponse:
        return OutagesResourceWithStreamingResponse(self._annotations.outages)


class AsyncAnnotationsResourceWithStreamingResponse:
    def __init__(self, annotations: AsyncAnnotationsResource) -> None:
        self._annotations = annotations

        self.list = async_to_streamed_response_wrapper(
            annotations.list,
        )

    @cached_property
    def outages(self) -> AsyncOutagesResourceWithStreamingResponse:
        return AsyncOutagesResourceWithStreamingResponse(self._annotations.outages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TunnelListParams"]


class TunnelListParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    exclude_prefix: str

    existed_at: str
    """
    If provided, include only resources that were created (and not deleted) before
    this time. URL encoded.
    """

    include_prefix: str

    is_deleted: bool
    """If `true`, only include deleted tunnels.

    If `false`, exclude deleted tunnels. If empty, all tunnels will be included.
    """

    name: str
    """A user-friendly name for the tunnel."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of results to display."""

    status: Literal["inactive", "degraded", "healthy", "down"]
    """The status of the tunnel.

    Valid values are `inactive` (tunnel has never been run), `degraded` (tunnel is
    active and able to serve traffic but in an unhealthy state), `healthy` (tunnel
    is active and able to serve traffic), or `down` (tunnel can not serve traffic as
    it has no connections to the Cloudflare Edge).
    """

    tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]]
    """The types of tunnels to filter by, separated by commas."""

    uuid: str
    """UUID of the tunnel."""

    was_active_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    was_inactive_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["MembershipListParams", "Account"]


class MembershipListParams(TypedDict, total=False):
    account: Account

    direction: Literal["asc", "desc"]
    """Direction to order memberships."""

    name: str
    """Account name"""

    order: Literal["id", "account.name", "status"]
    """Field to order memberships by."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of memberships per page."""

    status: Literal["accepted", "pending", "rejected"]
    """Status of this membership."""


class Account(TypedDict, total=False):
    name: str
    """Account name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_transit.pcaps import ownership_create_params, ownership_validate_params
from ....types.magic_transit.pcaps.ownership import Ownership

__all__ = ["OwnershipResource", "AsyncOwnershipResource"]


class OwnershipResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OwnershipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OwnershipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OwnershipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OwnershipResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        destination_conf: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Ownership:
        """
        Adds an AWS or GCP bucket to use with full packet captures.

        Args:
          account_id: Identifier.

          destination_conf: The full URI for the bucket. This field only applies to `full` packet captures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pcaps/ownership", account_id=account_id),
            body=maybe_transform({"destination_conf": destination_conf}, ownership_create_params.OwnershipCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Ownership]._unwrapper,
            ),
            cast_to=cast(Type[Ownership], ResultWrapper[Ownership]),
        )

    def delete(
        self,
        ownership_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes buckets added to the packet captures API.

        Args:
          account_id: Identifier.

          ownership_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ownership_id:
            raise ValueError(f"Expected a non-empty value for `ownership_id` but received {ownership_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/accounts/{account_id}/pcaps/ownership/{ownership_id}",
                account_id=account_id,
                ownership_id=ownership_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Ownership]:
        """
        List all buckets configured for use with PCAPs API.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pcaps/ownership", account_id=account_id),
            page=SyncSinglePage[Ownership],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Ownership,
        )

    def validate(
        self,
        *,
        account_id: str,
        destination_conf: str,
        ownership_challenge: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Ownership:
        """
        Validates buckets added to the packet captures API.

        Args:
          account_id: Identifier.

          destination_conf: The full URI for the bucket. This field only applies to `full` packet captures.

          ownership_challenge: The ownership challenge filename stored in the bucket.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pcaps/ownership/validate", account_id=account_id),
            body=maybe_transform(
                {
                    "destination_conf": destination_conf,
                    "ownership_challenge": ownership_challenge,
                },
                ownership_validate_params.OwnershipValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Ownership]._unwrapper,
            ),
            cast_to=cast(Type[Ownership], ResultWrapper[Ownership]),
        )


class AsyncOwnershipResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOwnershipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOwnershipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOwnershipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOwnershipResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        destination_conf: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Ownership:
        """
        Adds an AWS or GCP bucket to use with full packet captures.

        Args:
          account_id: Identifier.

          destination_conf: The full URI for the bucket. This field only applies to `full` packet captures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pcaps/ownership", account_id=account_id),
            body=await async_maybe_transform(
                {"destination_conf": destination_conf}, ownership_create_params.OwnershipCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Ownership]._unwrapper,
            ),
            cast_to=cast(Type[Ownership], ResultWrapper[Ownership]),
        )

    async def delete(
        self,
        ownership_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes buckets added to the packet captures API.

        Args:
          account_id: Identifier.

          ownership_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ownership_id:
            raise ValueError(f"Expected a non-empty value for `ownership_id` but received {ownership_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/accounts/{account_id}/pcaps/ownership/{ownership_id}",
                account_id=account_id,
                ownership_id=ownership_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Ownership, AsyncSinglePage[Ownership]]:
        """
        List all buckets configured for use with PCAPs API.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pcaps/ownership", account_id=account_id),
            page=AsyncSinglePage[Ownership],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Ownership,
        )

    async def validate(
        self,
        *,
        account_id: str,
        destination_conf: str,
        ownership_challenge: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Ownership:
        """
        Validates buckets added to the packet captures API.

        Args:
          account_id: Identifier.

          destination_conf: The full URI for the bucket. This field only applies to `full` packet captures.

          ownership_challenge: The ownership challenge filename stored in the bucket.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pcaps/ownership/validate", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "destination_conf": destination_conf,
                    "ownership_challenge": ownership_challenge,
                },
                ownership_validate_params.OwnershipValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Ownership]._unwrapper,
            ),
            cast_to=cast(Type[Ownership], ResultWrapper[Ownership]),
        )


class OwnershipResourceWithRawResponse:
    def __init__(self, ownership: OwnershipResource) -> None:
        self._ownership = ownership

        self.create = to_raw_response_wrapper(
            ownership.create,
        )
        self.delete = to_raw_response_wrapper(
            ownership.delete,
        )
        self.get = to_raw_response_wrapper(
            ownership.get,
        )
        self.validate = to_raw_response_wrapper(
            ownership.validate,
        )


class AsyncOwnershipResourceWithRawResponse:
    def __init__(self, ownership: AsyncOwnershipResource) -> None:
        self._ownership = ownership

        self.create = async_to_raw_response_wrapper(
            ownership.create,
        )
        self.delete = async_to_raw_response_wrapper(
            ownership.delete,
        )
        self.get = async_to_raw_response_wrapper(
            ownership.get,
        )
        self.validate = async_to_raw_response_wrapper(
            ownership.validate,
        )


class OwnershipResourceWithStreamingResponse:
    def __init__(self, ownership: OwnershipResource) -> None:
        self._ownership = ownership

        self.create = to_streamed_response_wrapper(
            ownership.create,
        )
        self.delete = to_streamed_response_wrapper(
            ownership.delete,
        )
        self.get = to_streamed_response_wrapper(
            ownership.get,
        )
        self.validate = to_streamed_response_wrapper(
            ownership.validate,
        )


class AsyncOwnershipResourceWithStreamingResponse:
    def __init__(self, ownership: AsyncOwnershipResource) -> None:
        self._ownership = ownership

        self.create = async_to_streamed_response_wrapper(
            ownership.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            ownership.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            ownership.get,
        )
        self.validate = async_to_streamed_response_wrapper(
            ownership.validate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource

__all__ = ["HistoryResource", "AsyncHistoryResource"]


class HistoryResource(SyncAPIResource):
    @cached_property
    def logs(self) -> LogsResource:
        return LogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> HistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return HistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return HistoryResourceWithStreamingResponse(self)


class AsyncHistoryResource(AsyncAPIResource):
    @cached_property
    def logs(self) -> AsyncLogsResource:
        return AsyncLogsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncHistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncHistoryResourceWithStreamingResponse(self)


class HistoryResourceWithRawResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

    @cached_property
    def logs(self) -> LogsResourceWithRawResponse:
        return LogsResourceWithRawResponse(self._history.logs)


class AsyncHistoryResourceWithRawResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

    @cached_property
    def logs(self) -> AsyncLogsResourceWithRawResponse:
        return AsyncLogsResourceWithRawResponse(self._history.logs)


class HistoryResourceWithStreamingResponse:
    def __init__(self, history: HistoryResource) -> None:
        self._history = history

    @cached_property
    def logs(self) -> LogsResourceWithStreamingResponse:
        return LogsResourceWithStreamingResponse(self._history.logs)


class AsyncHistoryResourceWithStreamingResponse:
    def __init__(self, history: AsyncHistoryResource) -> None:
        self._history = history

    @cached_property
    def logs(self) -> AsyncLogsResourceWithStreamingResponse:
        return AsyncLogsResourceWithStreamingResponse(self._history.logs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["OriginSteeringParam"]


class OriginSteeringParam(TypedDict, total=False):
    """Configures origin steering for the pool.

    Controls how origins are selected for new sessions and traffic without session affinity.
    """

    policy: Literal["random", "hash", "least_outstanding_requests", "least_connections"]
    """The type of origin steering policy to use.

    - `"random"`: Select an origin randomly.
    - `"hash"`: Select an origin by computing a hash over the CF-Connecting-IP
      address.
    - `"least_outstanding_requests"`: Select an origin by taking into consideration
      origin weights, as well as each origin's number of outstanding requests.
      Origins with more pending requests are weighted proportionately less relative
      to others.
    - `"least_connections"`: Select an origin by taking into consideration origin
      weights, as well as each origin's number of open connections. Origins with
      more open connections are weighted proportionately less relative to others.
      Supported for HTTP/1 and HTTP/2 connections.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ZoneHold"]


class ZoneHold(BaseModel):
    hold: Optional[bool] = None

    hold_after: Optional[str] = None

    include_subdomains: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["DHCPRelay"]


class DHCPRelay(BaseModel):
    server_addresses: Optional[List[str]] = None
    """List of DHCP server IPs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["CredentialSetEditResponse"]


class CredentialSetEditResponse(BaseModel):
    id: str
    """Credential set identifier."""

    name: str
    """Human-readable name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConnectionListParams"]


class ConnectionListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier"""

    direction: Literal["asc", "desc"]
    """The direction used to sort returned connections."""

    exclude_cdn_cgi: bool
    """
    When true, excludes connections seen in a `/cdn-cgi` path from the returned
    connections. The default value is true.
    """

    exclude_urls: str
    """
    Excludes connections whose URL contains one of the URL-encoded URLs separated by
    commas.
    """

    export: Literal["csv"]
    """Export the list of connections as a file, limited to 50000 entries."""

    hosts: str
    """
    Includes connections that match one or more URL-encoded hostnames separated by
    commas.

    Wildcards are supported at the start and end of each hostname to support starts
    with, ends with and contains. If no wildcards are used, results will be filtered
    by exact match
    """

    order_by: Literal["first_seen_at", "last_seen_at"]
    """The field used to sort returned connections."""

    page: str
    """The current page number of the paginated results.

    We additionally support a special value "all". When "all" is used, the API will
    return all the connections with the applied filters in a single page. This
    feature is best-effort and it may only work for zones with a low number of
    connections
    """

    page_url: str
    """
    Includes connections that match one or more page URLs (separated by commas)
    where they were last seen

    Wildcards are supported at the start and end of each page URL to support starts
    with, ends with and contains. If no wildcards are used, results will be filtered
    by exact match
    """

    per_page: float
    """The number of results per page."""

    prioritize_malicious: bool
    """When true, malicious connections appear first in the returned connections."""

    status: str
    """
    Filters the returned connections using a comma-separated list of connection
    statuses. Accepted values: `active`, `infrequent`, and `inactive`. The default
    value is `active`.
    """

    urls: str
    """
    Includes connections whose URL contain one or more URL-encoded URLs separated by
    commas.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.r2_data_catalog import maintenance_config_update_params
from ...types.r2_data_catalog.maintenance_config_get_response import MaintenanceConfigGetResponse
from ...types.r2_data_catalog.maintenance_config_update_response import MaintenanceConfigUpdateResponse

__all__ = ["MaintenanceConfigsResource", "AsyncMaintenanceConfigsResource"]


class MaintenanceConfigsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MaintenanceConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MaintenanceConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MaintenanceConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MaintenanceConfigsResourceWithStreamingResponse(self)

    def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        compaction: maintenance_config_update_params.Compaction | Omit = omit,
        snapshot_expiration: maintenance_config_update_params.SnapshotExpiration | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MaintenanceConfigUpdateResponse]:
        """Update the maintenance configuration for a catalog.

        This allows you to enable or
        disable compaction and adjust target file sizes for optimization.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          compaction: Updates compaction configuration (all fields optional).

          snapshot_expiration: Updates snapshot expiration configuration (all fields optional).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/maintenance-configs",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=maybe_transform(
                {
                    "compaction": compaction,
                    "snapshot_expiration": snapshot_expiration,
                },
                maintenance_config_update_params.MaintenanceConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MaintenanceConfigUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[MaintenanceConfigUpdateResponse]], ResultWrapper[MaintenanceConfigUpdateResponse]
            ),
        )

    def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MaintenanceConfigGetResponse]:
        """
        Retrieve the maintenance configuration for a specific catalog, including
        compaction settings and credential status.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/maintenance-configs",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MaintenanceConfigGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MaintenanceConfigGetResponse]], ResultWrapper[MaintenanceConfigGetResponse]),
        )


class AsyncMaintenanceConfigsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMaintenanceConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMaintenanceConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMaintenanceConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMaintenanceConfigsResourceWithStreamingResponse(self)

    async def update(
        self,
        bucket_name: str,
        *,
        account_id: str,
        compaction: maintenance_config_update_params.Compaction | Omit = omit,
        snapshot_expiration: maintenance_config_update_params.SnapshotExpiration | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MaintenanceConfigUpdateResponse]:
        """Update the maintenance configuration for a catalog.

        This allows you to enable or
        disable compaction and adjust target file sizes for optimization.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          compaction: Updates compaction configuration (all fields optional).

          snapshot_expiration: Updates snapshot expiration configuration (all fields optional).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/maintenance-configs",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=await async_maybe_transform(
                {
                    "compaction": compaction,
                    "snapshot_expiration": snapshot_expiration,
                },
                maintenance_config_update_params.MaintenanceConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MaintenanceConfigUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[MaintenanceConfigUpdateResponse]], ResultWrapper[MaintenanceConfigUpdateResponse]
            ),
        )

    async def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MaintenanceConfigGetResponse]:
        """
        Retrieve the maintenance configuration for a specific catalog, including
        compaction settings and credential status.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/maintenance-configs",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MaintenanceConfigGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MaintenanceConfigGetResponse]], ResultWrapper[MaintenanceConfigGetResponse]),
        )


class MaintenanceConfigsResourceWithRawResponse:
    def __init__(self, maintenance_configs: MaintenanceConfigsResource) -> None:
        self._maintenance_configs = maintenance_configs

        self.update = to_raw_response_wrapper(
            maintenance_configs.update,
        )
        self.get = to_raw_response_wrapper(
            maintenance_configs.get,
        )


class AsyncMaintenanceConfigsResourceWithRawResponse:
    def __init__(self, maintenance_configs: AsyncMaintenanceConfigsResource) -> None:
        self._maintenance_configs = maintenance_configs

        self.update = async_to_raw_response_wrapper(
            maintenance_configs.update,
        )
        self.get = async_to_raw_response_wrapper(
            maintenance_configs.get,
        )


class MaintenanceConfigsResourceWithStreamingResponse:
    def __init__(self, maintenance_configs: MaintenanceConfigsResource) -> None:
        self._maintenance_configs = maintenance_configs

        self.update = to_streamed_response_wrapper(
            maintenance_configs.update,
        )
        self.get = to_streamed_response_wrapper(
            maintenance_configs.get,
        )


class AsyncMaintenanceConfigsResourceWithStreamingResponse:
    def __init__(self, maintenance_configs: AsyncMaintenanceConfigsResource) -> None:
        self._maintenance_configs = maintenance_configs

        self.update = async_to_streamed_response_wrapper(
            maintenance_configs.update,
        )
        self.get = async_to_streamed_response_wrapper(
            maintenance_configs.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rum import (
    RUMResource,
    AsyncRUMResource,
    RUMResourceWithRawResponse,
    AsyncRUMResourceWithRawResponse,
    RUMResourceWithStreamingResponse,
    AsyncRUMResourceWithStreamingResponse,
)
from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .site_info import (
    SiteInfoResource,
    AsyncSiteInfoResource,
    SiteInfoResourceWithRawResponse,
    AsyncSiteInfoResourceWithRawResponse,
    SiteInfoResourceWithStreamingResponse,
    AsyncSiteInfoResourceWithStreamingResponse,
)

__all__ = [
    "SiteInfoResource",
    "AsyncSiteInfoResource",
    "SiteInfoResourceWithRawResponse",
    "AsyncSiteInfoResourceWithRawResponse",
    "SiteInfoResourceWithStreamingResponse",
    "AsyncSiteInfoResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "RUMResource",
    "AsyncRUMResource",
    "RUMResourceWithRawResponse",
    "AsyncRUMResourceWithRawResponse",
    "RUMResourceWithStreamingResponse",
    "AsyncRUMResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["CustomUpdateParams"]


class CustomUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    bucket_name: Required[str]
    """Name of the bucket."""

    ciphers: SequenceNotStr[str]
    """An allowlist of ciphers for TLS termination.

    These ciphers must be in the BoringSSL format.
    """

    enabled: bool
    """Whether to enable public bucket access at the specified custom domain."""

    min_tls: Annotated[Literal["1.0", "1.1", "1.2", "1.3"], PropertyInfo(alias="minTLS")]
    """Minimum TLS Version the custom domain will accept for incoming connections.

    If not set, defaults to previous value.
    """

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""
